@@ -2335,7 +2335,8 @@ gdk_window_shape_combine_mask (GdkWindow *window,
 			       GdkBitmap *mask,
 			       gint x, gint y)
 {
-  g_return_if_fail (window != NULL);
+  GdkWindowObject *private = (GdkWindowObject *)window;
+
   g_return_if_fail (GDK_IS_WINDOW (window));
 
   if (!mask)
@@ -2343,6 +2344,8 @@ gdk_window_shape_combine_mask (GdkWindow *window,
       GDK_NOTE (MISC, g_print ("gdk_window_shape_combine_mask: %p: none\n",
 			       GDK_WINDOW_HWND (window)));
       SetWindowRgn (GDK_WINDOW_HWND (window), NULL, TRUE);
+
+      private->shaped = FALSE;
     }
   else
     {
@@ -2370,6 +2373,8 @@ gdk_window_shape_combine_mask (GdkWindow *window,
 	}
       
       SetWindowRgn (GDK_WINDOW_HWND (window), hrgn, TRUE);
+
+      private->shaped = TRUE;
     }
 }
 
